/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "particlesystem.h"

ParticleSystem::ParticleSystem(GLuint _texture, 
                               const Vec3 & _pos,
                               int _max_particle,
                               int create_at_start,
                               float _minlife, float _maxlife, 
							   bool _create_new, 
                               float _new_per_sec, 
                               const Vec3 & _velocity, 
                               float _minvar, float _maxvar,
                               bool _slow_down,
                               float _size, 
                               const Vec4 & _start_color, 
                               const Vec4 & _end_color,
                               float _time) :
Effect(_pos),
texture(_texture),
max_particle(_max_particle),
minlife(_minlife), maxlife(_maxlife),
create_new(_create_new),
new_per_sec(_new_per_sec),
new_particle(0.0f),
velocity(_velocity),
minvar(_minvar), maxvar(_maxvar),
slowdown(_slow_down),
size(_size),
start_color(_start_color),
end_color(_end_color), 
particle_arr(NULL),
index_arr(NULL)
{
	particle_arr = new Particle[max_particle];	
	index_arr = new int[max_particle];
	
	create_at_start = min(max_particle, create_at_start);
	new_particle = (float) create_at_start;
	num_active = 0;	
		
	for (int i = 0; i < max_particle; i++) {
			particle_arr[i].active = false;
	}	
	
	last_update = _time;
}

ParticleSystem::~ParticleSystem() {
	delete[] particle_arr;
	delete[] index_arr;
}

void ParticleSystem::process(float time) {
	Particle *p;
	float t;
	int j = 0;
	float deltat = time - last_update;
	last_update = time;
	
	if (create_new)
		new_particle += new_per_sec * deltat;
	
	for (int i = 0; i < max_particle; i++) {
		p = particle_arr + i;
		
		// particle die
		if (p->start_time + p->life_time < time) {
			p->active = false;
		}
		
		if (!p->active && new_particle < 1.0f) {
			continue;
		}
		
		if (!p->active) {
			p->pos = pos;
			p->vel = velocity + Vec3::randomUnit() * frand(minvar, maxvar);
			p->start_time = time;
			p->life_time = frand(minlife, maxlife);
			p->life_time = max(0.001f, p->life_time);
			p->active = true;
			new_particle -= 1.0f;
			t = (time - p->start_time) / p->life_time;
		} else {
			t = (time - p->start_time) / p->life_time;
			if (slowdown) 
				p->pos += p->vel * deltat * (1-t);
			else
				p->pos += p->vel * deltat;
		}
		
		p->color = (start_color * (1-t)) + (end_color * t);
		index_arr[j] = i;
		j++;
	}
	
	num_active = j;
}

void ParticleSystem::render(const Vec3 & campos) {
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexEnvf(GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE);
	
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_VERTEX_ARRAY);		
	glColorPointer(4, GL_FLOAT, sizeof(Particle), &particle_arr[0].color);
	glVertexPointer(3, GL_FLOAT, sizeof(Particle), &particle_arr[0].pos);
	
	// pointparameter in cometenscene->init()

	glEnable(GL_POINT_SPRITE_ARB);
	glPointSize(size);

	
	glDrawElements(GL_POINTS, num_active, GL_UNSIGNED_INT, index_arr);
	
	glDisableClientState(GL_COLOR_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
	
	glDisable(GL_POINT_SPRITE_ARB);
	glTexEnvf(GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_FALSE);
	
	glDisable(GL_BLEND);
}

void ParticleSystem::stop() {
	create_new = false;
}

bool ParticleSystem::done() {
	return (!create_new) && (num_active == 0);
}
