/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <cmath>
using namespace std;

#include <math.h>

#include "vec.h"
#include "cometen_math.h"

const Vec2 Vec2::ZERO(0.0f, 0.0f);
const Vec2 Vec2::X(1.0f, 0.0f);
const Vec2 Vec2::Y(0.0f, 1.0f);

Vec3 Vec3::randomUnit() {
	Vec3 newvec(frand(-1.0f,1.0f), 
		frand(-1.0f,1.0f), 
		frand(-1.0f,1.0f));
	
	float l = newvec.getLength();
	if (l == 0.0f) {
		return Vec3(1.0f, 0.0f, 0.0f);
	}
	else {
		newvec /= l;	
		return newvec;
	}	
}

void Vec3::reflect(const Vec3 & normal) {
	float a = dot(normal) * 2;
	
	x = x - normal.x * a; 
	y = y - normal.y * a;
	z = z - normal.z * a;
}

const Vec3 Vec3::ZERO(0.0f, 0.0f, 0.0f);
const Vec3 Vec3::X(1.0f, 0.0f, 0.0f);
const Vec3 Vec3::Y(0.0f, 1.0f, 0.0f);
const Vec3 Vec3::Z(0.0f, 0.0f, 1.0f);


const Vec4 Vec4::ZERO(0.0f, 0.0f, 0.0f, 0.0f);
const Vec4 Vec4::X(1.0f, 0.0f, 0.0f, 0.0f);
const Vec4 Vec4::Y(0.0f, 1.0f, 0.0f, 0.0f);
const Vec4 Vec4::Z(0.0f, 0.0f, 1.0f, 0.0f);
const Vec4 Vec4::W(0.0f, 0.0f, 0.0f, 1.0f);

// ===============================  Vec2  ==================================

bool Vec2::operator==(const Vec2 & b) const {
	return (x == b.x) && (y == b.y);
}

bool Vec2::operator!=(const Vec2 & b) const {
	return (x != b.x) || (y != b.y);
}


void Vec2::operator+=(const Vec2 & b) {
	x += b.x;
	y += b.y;
}

void Vec2::operator-=(const Vec2 & b) {
	x -= b.x;
	y -= b.y;	
}

void Vec2::operator*=(float b) {
	x *= b;
	y *= b;
}

void Vec2::operator/=(float b) {
	x /= b;
	y /= b;
}

Vec2 Vec2::operator+(const Vec2 & b) const {
	return Vec2(x + b.x, y + b.y);
}

Vec2 Vec2::operator-(const Vec2 & b) const {
	return Vec2(x - b.x, y - b.y);
}

Vec2 Vec2::operator-() const {
	return Vec2(-x, -y);
}

Vec2 Vec2::operator*(float b) const {
	return Vec2(x*b, y*b);
}
Vec2 Vec2::operator/(float b) const {
	return Vec2(x/b, y/b);
}

float Vec2::dot(const Vec2 &b) const {
	return x*b.x + y*b.y;
}

float Vec2::getLength() const {
	return sqrtf(x*x + y*y);
}

float Vec2::getSquareLength() const {
	return x*x + y*y;
}

void Vec2::normalize() {
	float l = getLength();
	x /= l;
	y /= l;
}

// ===============================  Vec3  ==================================

bool Vec3::operator==(const Vec3 & b) const {
	return (x == b.x) && (y == b.y) && (z == b.z);
}

bool Vec3::operator!=(const Vec3 & b) const {
	return (x != b.x) || (y != b.y) || (z != b.z);
}

void Vec3::operator+=(const Vec3 & b) {
	x += b.x;
	y += b.y;
	z += b.z;
}

void Vec3::operator-=(const Vec3 & b) {
	x -= b.x;
	y -= b.y;
	z -= b.z;
}

void Vec3::operator*=(float b) {
	x *= b;
	y *= b;
	z *= b;
}

void Vec3::operator/=(float b) {
	x /= b;
	y /= b;
	z /= b;
}

Vec3 Vec3::operator+(const Vec3 & b) const {
	return Vec3(x + b.x, y + b.y, z + b.z);
}

Vec3 Vec3::operator-(const Vec3 & b) const {
	return Vec3(x - b.x, y - b.y, z - b.z);
}

Vec3 Vec3::operator-() const {
	return Vec3(-x, -y, -z);
}

Vec3 Vec3::operator*(float b) const {
	return Vec3(x*b, y*b, z*b);
}

Vec3 Vec3::operator/(float b) const {
	return Vec3(x/b, y/b, z/b);
}

float Vec3::dot(const Vec3 &b) const {
	return x*b.x + y*b.y + z*b.z;
}

Vec3 Vec3::cross(const Vec3 &b) const {
	return Vec3(y*b.z - z*b.y, z*b.x - x*b.z, x*b.y - y*b.x);
}

float Vec3::getLength() const {
	return sqrtf(x*x + y*y + z*z);
}

float Vec3::getSquareLength() const {
	return x*x + y*y + z*z;
}

void Vec3::normalize() {
	float a = 1 / getLength();
	x *= a;
	y *= a;
	z *= a;
}

// ===============================  Vec4  ==================================

bool Vec4::operator==(const Vec4 & b) const {
	return (x == b.x) && (y == b.y) && (z == b.z) && (w == b.w);
}

bool Vec4::operator!=(const Vec4 & b) const {
	return (x != b.x) || (y != b.y) || (z != b.z) || (w != b.w);
}

void Vec4::operator+=(const Vec4 & b) {
	x += b.x;
	y += b.y;
	z += b.z;
	w += b.w;
}

void Vec4::operator-=(const Vec4 & b) {
	x -= b.x;
	y -= b.y;
	z -= b.z;
	w -= b.w;
}

void Vec4::operator*=(float b) {
	x *= b;
	y *= b;
	z *= b;
	w *= b;
}

void Vec4::operator/=(float b) {
	x /= b;
	y /= b;
	z /= b;
	w /= b;
}

Vec4 Vec4::operator+(const Vec4 & b) const {
	return Vec4(x + b.x, y + b.y, z + b.z, w + b.w);
}

Vec4 Vec4::operator-(const Vec4 & b) const {
	return Vec4(x - b.x, y - b.y, z - b.z, w - b.w);
}

Vec4 Vec4::operator-() const {
	return Vec4(-x, -y, -z, -w);
}

Vec4 Vec4::operator*(float b) const {
	return Vec4(x*b, y*b, z*b, w*b);
}

Vec4 Vec4::operator/(float b) const {
	return Vec4(x/b, y/b, z/b, w/b);
}

float Vec4::dot(const Vec4 &b) const {
	return x*b.x + y*b.y + z*b.z + w*b.w;
}

float Vec4::getLength() const {
	return sqrtf(x*x + y*y + z*z + w*w);
}

float Vec4::getSquareLength() const {
	return x*x + y*y + z*z + w*w;
}

void Vec4::normalize() {
	float l = getLength();
	x /= l;
	y /= l;
	z /= l;
	w /= l;
}

